-- This does not handle escaped commas with quotes!

local M = {}

function split(s,delimiter)
	delimiter = delimiter or '%s'
	local t={}
	local i=1
	for str in string.gmatch(s .. delimiter, '([^' .. delimiter .. ']*)' .. delimiter) do
		t[i] = str
		i = i + 1
	end
	return t
end

local function read_file(file)
	local f = assert(io.open(file, "rb"))
	local content = f:read("*all")
	f:close()
	return content
end

local function normalize_line_endings(file)
	return string.gsub(file, '\r\n?', '\n')
end

function M.parse_csv(csv_file, delimiter, header)
	delimiter = delimiter or ","
	local csv_table = {}
	csv_file = normalize_line_endings(csv_file)
	for line in csv_file:gmatch("(.-)\n") do
		table.insert(csv_table, split(line, delimiter))
	end
	if header then
		local keyed_table = {}
		local header_line = table.remove(csv_table, 1)
		for _,line in ipairs(csv_table) do
			local new_line = {}
			for i,key in ipairs(header_line) do
				new_line[key] = line[i]
			end
			table.insert(keyed_table, new_line)
		end
		return keyed_table
	end
	return csv_table
end


function M.load_file(file_path, delimiter, header)
	local data = read_file(file_path)
	if data then
		local data_table = M.parse_csv(data, delimiter, header)
		return data_table
	else
		print(error)
		return nil
	end
end

function M.load_resource(resource_path, delimiter, header)
	local data, error = sys.load_resource(resource_path)
	if data then
		local data_table = M.parse_csv(data, delimiter, header)
		return data_table
	else
		print(error)
		return nil
	end	
end

return M