local richtext = require "richtext.richtext"
local utf8 =require("richtext.utf8")

local M = {}

local template = ""
local current_timer = nil
local name_timer = nil
local current_string = nil
local current_name = nil
local words, metrics = nil
local completion_fn = nil
local cursor, next_icon = nil
local names = {}
local confirm_button, cancel_button = nil
local show_choices = false

local font_settings = {
	width = 720,
	color = vmath.vector4(0, 0, 0, 1.0),
	align = richtext.ALIGN_LEFT,
	valign = richtext.VALIGN_TOP
}

M.set_buttons = function(opts)
	confirm_button = opts["confirm"]
	cancel_button = opts["cancel"]
end

M.reset = function()
	if current_timer ~= nil then
		timer.cancel(current_timer)
	end
	if name_timer ~= nil then
		timer.cancel(name_timer)
	end
	
	if words ~= nil then
		richtext.remove(words)
	end
	current_timer, name_timer, current_string, current_name, words, metrics = nil
end

M.finish = function()
	gui.set_enabled(cursor, false)
	if show_choices then
		gui.set_enabled(confirm_button, true)
		gui.set_enabled(cancel_button, true)
	else
		gui.set_enabled(next_icon, true)
		gui.play_flipbook(next_icon, "next")
	end
	completion_fn()
	completion_fn = nil
end

M.set_name = function(speaker, name, replace)
	if not speaker then return end

	local name_label = gui.get_node(template.."/name_label")

	local oldname = ""
	if speaker then
		if speaker == "NARRATOR" then
			name = ""
		elseif speaker == "PLAYER" then
			name = "Player"
		else
			oldname = names[speaker]
			if name and name ~= "" then
				names[speaker] = name
			else
				-- no name; reuse old name
				name = names[speaker]
				if name and name ~= "" then
					replace = false
				end
			end
		end
	end

	if replace then
		print("replacing '"..oldname.."' with '"..name.."'")
		local t = -1

		gui.set_text(name_label, oldname)
		name_timer = timer.delay(0.1, true, function(self, handle, time_elapsed)
			t = t + 1
			if t <= #oldname then
				gui.set_text(name_label, string.sub(oldname, 1, #oldname - t))
				return
			end
			local i = t - #oldname
			if i > 0 and i <= #name then
				gui.set_text(name_label, string.sub(name, 1, i))
				return
			end
			
			timer.cancel(handle)
		end)
	else
		gui.set_text(name_label, name)
	end
end

M.show_text = function(speaker, name, text, voice, choices, completion)
	print("showing: "..text)
	M.reset()

	local replace = false
	name = name or ""
	if speaker then
		if names[speaker] and names[speaker] ~= name then
			replace = true
		end
	end

	local is_narration = speaker == nil
	if is_narration then
		template = "dialogue - narration"
	else
		template = "dialogue"
	end

	M.set_name(speaker, name, replace)

	current_string = text
	current_name = name
	completion_fn = completion
	
	local label = gui.get_node(template.."/text")
	
	gui.set_text(label, "")
	local settings = font_settings
	settings["parent"] = label

	cursor = gui.get_node(template.."/cursor")
	gui.set_parent(cursor, label)
	gui.set_enabled(cursor, true)
	gui.play_flipbook(cursor, "cursor")

	next_icon = gui.get_node(template.."/next_icon")
	gui.set_enabled(next_icon, false)

	local box = gui.get_node(template.."/dialogue")
	gui.set_enabled(box, true)

	show_choices = choices ~= nil
	if show_choices then
		gui.set_text(gui.get_node(template.."/confirm text"), choices[1].text)
		gui.set_text(gui.get_node(template.."/cancel text"), choices[2].text)

	end
	gui.set_enabled(confirm_button, false)
	gui.set_enabled(cancel_button, false)

	gui.set_scale(box, vmath.vector3(1, 0, 1))
	gui.animate(box, gui.PROP_SCALE, vmath.vector3(1, 1, 1), gui.EASING_OUTSINE, 0.33, 0, function()
		words, metrics = richtext.create(current_string, "rounded-mplus", settings)
		words = words
		local t = 0
		richtext.truncate(words, t, { words = false })
		current_timer = timer.delay(0.05, true, function(self, handle, time_elapsed)
			t = t + 1
			local last_word = richtext.truncate(words, t, { words = false })
			-- cursor
			local pos = vmath.vector3(settings.position)
			if last_word then
				pos = gui.get_position(last_word.node)
				pos.x = pos.x + last_word.metrics.width + 8
				pos.y = pos.y - 12
			end
			gui.set_position(cursor, pos)
			-- gui.set_enabled(cursor, not gui.is_enabled(cursor))

			if voice ~= null then
				if utf8.sub(text, t, t) ~= " " then
					local bark = voice[math.random(#voice)]
					sound.play(bark, { gain = 0.5 })
				end
			end

			if t == utf8.len(current_string) then
				current_timer = nil
				timer.cancel(handle)
			else
				return
			end
			
			M.finish()
		end)
	
	end)
end

M.skip = function()
	if current_timer ~= nil then
		richtext.truncate(words, utf8.len(current_string), { words = false })
		timer.cancel(current_timer)
		current_timer = nil
		M.finish()
	end
	if name_timer ~= nil then
		gui.set_text(gui.get_node("dialogue/name_label"), current_name)
		timer.cancel(name_timer)
		name_timer = nil
	end
end

M.dismiss = function(completion)
	local box = gui.get_node(template.."/dialogue")

	gui.set_scale(box, vmath.vector3(1, 1, 1))
	gui.animate(box, gui.PROP_SCALE, vmath.vector3(1, 0, 1), gui.EASING_OUTSINE, 0.33, 0, function()
		gui.set_enabled(box, false)
		completion()
	end)
end

M.typing = function()
	return current_timer ~= nil
end

M.game_over = function()
	gui.set_enabled(next_icon, true)
	gui.play_flipbook(next_icon, "end")
end

return M