local camera = require "orthographic/camera"

function get_scaling_factor()
	if not defos then return 1 end
	local displays = defos.get_displays()
	if not displays or #displays == 0 then
		return 1
	end
	local main_display = displays[1]
	return main_display.mode and main_display.mode.scaling_factor or 1
end

function query_displays()
	local dpi_ratio = get_scaling_factor()
	local width, height = tonumber(sys.get_config("display.width")), tonumber(sys.get_config("display.height"))
	local window_width, window_height = window.get_size()
	print("screen scaling factor is "..dpi_ratio)
	print("game dim    : "..width.." x "..height)
	print("display dim : "..window_width.." x "..window_height)
end

local function convert_pt(point, source, destination)
	local s = vmath.vector3(0, 0, 0)
	s.x = ((point.x - source.x) / source.w) * destination.w + destination.x
	s.y = ((point.y - source.y) / source.h) * destination.h + destination.y
	return s
end

local function recenter(node, width, height)
	local center = vmath.vector3(0, 0, 0)
	center.x = math.floor(width / 2)
	center.y = math.floor(height / 2)
	gui.set_position(node, center)
	-- gui.set_position(blockout, center)
end

local function center_rect(a, b)
	local x = (b.w - a.w) / 2
	local y = (b.h - a.h) / 2
	a.x = x
	a.y = y
	return a
end

function convert_action_to_world(action)
	if action.x and action.y then
		local width, height = tonumber(sys.get_config("display.width")), tonumber(sys.get_config("display.height"))
		local window_width, window_height = window.get_size()
		local scale_w, scale_h = window_width / width, window_height / height
		local zoom = math.min(scale_w, scale_h)

		zoom = math.max(1, math.floor(zoom))
		
		local offsetx = (window_width / zoom - width) / (2 * scale_w)
		local offsety = (window_height / zoom - height) / (2 * scale_h)
		
		local p = vmath.vector3(action.x, action.y, 0)
		p = p / zoom

		action.x = p.x - offsetx
		action.y = p.y - offsety

		return action
	end
	return action
end

