local M = {}

-- there's no builtin way to get the layers of a tilemap, so...
local function layer_exists(map_id, layer)
	if pcall(tilemap.set_visible, map_id, layer, true) then
		return true
	else
		print("layer "..layer.." does not exist on "..map_id)
		return false
	end
end

function M.iterate_layers(map_id, layer, lambda)
	if layer_exists(map_id, layer) then
		local origin_x,origin_y, width,height = tilemap.get_bounds(map_id)
		for x=origin_x,width do
			for y=origin_y,height do
				local obj_id = tilemap.get_tile(map_id, layer, x, y)
				if obj and obj_id ~= 0 then lambda(obj_id, x, y) end
			end
		end
	end
end

function M.iterate_maps(map_collection, map_name, sheets, lambda)
	-- dialogue:/collection0/tilemaps
	--                      /tilemaps#Japanese City.tilemap-control sprites
	
	for _, sheet in ipairs(sheets) do
		local map_id = msg.url(nil, map_collection, map_name..".tilemap-"..sheet)
		lambda(map_id)
	end
end

local layers = {
	["control sprites"] = {"Zones"}, 
	["Japanese City - building"] = {"3rd Floor", "2nd Floor", "1st Floor"}, 
	["Japanese City - ground"] = {"Ground"}, 
	["Japanese City - objects"] = {"2nd Floor", "Props - 1st Floor"}
}

function M.hard_iterate_layers(map_collection, map_name, incl_layers, lambda)
	for sheet, sublayers in ipairs(layers) do
		local map_id = msg.url(nil, map_collection, map_name..".tilemap-"..sheet)
		for _,layer in ipairs(sublayers) do
			if table.contains(incl_layers, layer) then
				local origin_x,origin_y, width,height = tilemap.get_bounds(map_id)
				for x=origin_x,width do
					for y=origin_y,height do
						local obj_id = tilemap.get_tile(map_id, layer, x, y)
						if obj and obj_id ~= 0 then lambda(obj_id, x, y) end
					end
				end
			end
		end
	end
end

return M