local items = require "main/items"

require "main/util"

local M = {}

local random_item = function()
	local keys = {}
	for k,_ in pairs(items) do
		table.insert(keys, k)
	end

	local item = items[table.random(keys)]
	return item
end

local new_beat = function(speaker, text, replies)
	local beat = {
		["text"] = text
	}
	if speaker then
		beat["speaker"] = speaker
	end
	if replies then
		beat["reply"] = replies
	end
	return beat
end

local new_check_beat = function(speaker, pass_text, fail_text, check)
	local beat = {
		["check"] = check,
		["pass_text"] = pass_text,
		["fail_text"] = fail_text
	}
	return beat
end

local new_reply = function(text, id)
	local reply = {
		["text"] = text,
		["goto"] = id
	}
	return reply
end

local dialogue_with_prompt = function(prompt, item)
	local d = {}
	d["1"] = {
		new_beat(nil, prompt, {
			new_reply("はい", "yes"),
			new_reply("いいえ", "no")
		})
	}
	d["yes"] = {
		-- Oh, thank you.
		new_beat(nil, "ああ、ありがとう。", {
			-- here it is
			new_reply("どうぞ", "give_item"),
			-- I don't have it
			new_reply("持ってない", "find_item")
		})
	}
	d["no"] = {
		-- Oh, maybe some other time.
		new_beat(nil, "ああ、多分また別の時。")
	}
	d["give_item"] = {
		-- Thanks!  /  No, that's not it.
		new_check_beat(nil, "ありがとう！", "いいえ、それは違います。", item.id)
	}
	d["find_item"] = {
		-- Thanks, come back when you find it!
		new_beat(nil, "ありがとう、見つけたら戻ってきてください！")
	}
	
	return d
end

local dialogue_for_item = function(item)
	local dialogues = {}
	dialogues[422] = function()
		-- Could you get me a soda from the red vending machine?
		return dialogue_with_prompt("赤い自動販売機からソーダをもらえますか？", item)
	end
	dialogues[423] = function()
		-- I'd like a pack of smokes from the white vending machine.
		return dialogue_with_prompt("白い自動販売機からの煙のパックが欲しいのですが。", item)
	end

	dialogues[31] = function()
		-- I don't want to walk. I want a bicycle.
		return dialogue_with_prompt("歩きたくない。 自転車が欲しいです。", item)
	end

	dialogues[32] = function()
		-- I am thirsty. May I get a drink from the fountain?
		return dialogue_with_prompt("のどが渇きました。 噴水から飲み物をもらってもよろしいですか。", item)
	end

	dialogues[47] = function()
		-- I love daffodils. Can I get a daffodil?
		return dialogue_with_prompt("水仙が大好きです。 水仙をもらえますか？", item)
	end

	dialogues[63] = function()
		-- Please, could you pick me some tulips?
		return dialogue_with_prompt("チューリップを選んでいただけませんか。", item)
	end

	dialogues[64] = function()
		-- I want to climb a pine tree. Do you know where one is?
		return dialogue_with_prompt("松に登りたいです。 どこにあるか知っていますか？", item)
	end

	

	return dialogues[item.id]()
end

-------

M.new_quest = function()
	local quest = {}
	quest.item = random_item()
	quest.dialogue = dialogue_for_item(quest.item)
	return quest
end

return M