function palette(n)
	local p = {
		[ 0] = "000000",
		[ 1] = "222034",
		[ 2] = "323c39",
		[ 3] = "595652",
		[ 4] = "3f3f74",
		[ 5] = "847e87",
		[ 6] = "37946e",
		[ 7] = "8f974a",
		[ 8] = "9badb7",
		[ 9] = "d77bba",
		[10] = "d9a066",
		[11] = "d95763",
		[12] = "5fcde4",
		[13] = "eec39a",
		[14] = "cbdbfc",
		[15] = "ffffff"
	}
	return p[n] or "ffff00"
end

table.filter = function(t, f)
	local out = {}
  
	for _,v in pairs(t) do
	  if f(v) then table.insert(out, v) end
	end
  
	return out
end

table.contains = function(t, v)
	for _,e in pairs(t) do
		if v == e then return true end
	end
	return false
end

table.random = function(t)
	return t[math.random(#t)]
end

table.shuffle = function(t)
	shuffled = {}
	for i, v in ipairs(t) do
		local pos = math.random(1, #shuffled+1)
		table.insert(shuffled, pos, v)
	end
	return shuffled
end

if not table.unpack then
	table.unpack = unpack
end

table.copy = function(orig)
	local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in next, orig, nil do
            copy[table.copy(orig_key)] = table.copy(orig_value)
        end
        setmetatable(copy, table.copy(getmetatable(orig)))
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end

table.update = function(a, b)
	local c = table.copy(a)
	for k,v in pairs(b) do
		c[k] = v
	end
	return c
end

function default_value(x, default)
	local result
	if x ~= nil then 
		return x
	else
		return default
	end
end

local IS_BUNDLE = nil

function is_bundle()
	if IS_BUNDLE ~= nil then return IS_BUNDLE end
	if sys.get_application_path():match(".*%/Defold%.app") ~= nil then
		IS_BUNDLE = false
		return IS_BUNDLE
	end

	local project_path = sys.get_application_path() .. package.config:sub(1,1) .. "game.projectc"
	print("checking "..project_path)
	local file = io.open(project_path, "r")
	if file then
		file:close()
		print("IS BUNDLE")
		IS_BUNDLE = true
		return IS_BUNDLE
	else
		project_path = sys.get_application_path() .. "/Contents/Resources/" .. "game.projectc"
		print("checking "..project_path)
		local file = io.open(project_path, "r")
		if file then
			file:close()
			print("IS BUNDLE")
			IS_BUNDLE = true
			return IS_BUNDLE
		end
	end
	
	print("NOT BUNDLE")
	IS_BUNDLE = false
	return IS_BUNDLE
end

function is_editor()
	return not is_bundle()
end