local machine = require "main/statemachine"
local items = require "main/items"
local quests = require "main/quests"
local map = require "main/map"

local M = {}

local active_quests = {}
M.TILE_SIZE = 32

local fsm = machine.create({
	initial = "move",
	events = {
		{ name = "interact", from = "move", to = "interaction" },
		{ name = "back", from = "interaction", to = "move" },
		{ name = "inv", from = "move", to = "inventory" },
		{ name = "back", from = "inventory", to = "move" }
	},
	callbacks = {
		
	}
})

local function interact_with(n, x, y)
	local obj = items[n]
	if obj then
		local d = {}
		d["1"] = {
			{
				["text"] = obj.name.."\n"..obj.english.."\n"..obj.desc
			}
		}
		msg.post("/go#monarch", "dialogue", d)
		fsm:interact()
		return true
	end
end

local function tile_at(x, y)
	local sheets = {"control sprites", "Japanese City - building", "Japanese City - ground", "Japanese City - objects"}
	local layers = {"3rd Floor", "2nd Floor", "Props - 1st Floor", "1st Floor", "Ground"}
	map.iterate_maps(M.current_map, "Japanese City", sheets, function(map_id)
		for _, layer in ipairs(layers) do
			map.iterate_layers(map_id, layer, function(obj_id, x, y)
				print("got obj = "..obj)
				return obj
			end)
		end
	end)
	return nil
end

local function actor_cast(x, y, dx, dy)
	local r_start = vmath.vector3(x, y, 1.5)
	local r_end = vmath.vector3(x + M.TILE_SIZE * dx, y + M.TILE_SIZE * dy, 1.5)
	local groups = { hash("actors") }

	local result = physics.raycast(r_start, r_end, groups)
	if result then
		-- act on the hit (see 'ray_cast_response' message for all values)
		print("ray hit "..result.id)
		return result
	end
end


-------------

function M.get_state()
	return fsm.current
end

function M.is_interacting()
	return fsm.current == "interaction"
end

function M.can_move()
	return fsm.current == "move"
end

function M.interact(player_state)
	local dx, dy = 0, 0
	if player_state == "wc_north" or player_state == "idle_north" then
		print("interact N")
		dy = 1
	elseif player_state == "wc_south" or player_state == "idle_south" then
		print("interact S")
		dy = -1
	elseif player_state == "wc_east" or player_state == "idle_east" then
		print("interact E")
		dx = 1
	elseif player_state == "wc_west" or player_state == "idle_west" then
		print("interact W")
		dx = -1
	elseif M.is_interacting(player_state) then
		msg.post("/go#monarch", "continue")
		return false
	else
		print("can't interact? ... " .. player_state)
		return false
	end

	local pos = go.get_position(hash("/Player"))
	local tile_x, tile_y = math.ceil(pos.x / M.TILE_SIZE) + dx, math.ceil(pos.y / M.TILE_SIZE + dy)
	print("interact @ "..tile_x..","..tile_y)

	local success = nil
	local obj = tile_at(tile_x, tile_y)
	if obj and obj ~= 0 then
		print("obj = "..obj)
		success = interact_with(obj, tile_x, tile_y)
		if success then
			M.selected_item = obj
			msg.post("/go#monarch", "selected_item", { t = obj })
			return success
		end
	end

	local actor = actor_cast(pos.x, pos.y, dx, dy)
	if actor then
		if not active_quests[actor.id] then
			active_quests[actor.id] = quests.new_quest()
		end
		
		local quest = active_quests[actor.id]
		
		msg.post("/go#monarch", "dialogue", quest.dialogue)
		fsm:interact()
		success = true
	end
	return success
end

function M.show_inventory()
	if fsm.current == "move" then
		msg.post("/go#monarch", "toggle_inventory")
		fsm:inv()
	elseif fsm.current == "inventory" then
		msg.post("/go#monarch", "toggle_inventory")
		fsm:back()
	end
end

function M.resume()
	print("resume...")
	fsm:back()
end


return M